@@ -2,7 +2,7 @@ import argparse
 import sys
 
 from tfkit.utility import tok
-from tqdm import tqdm
+from tqdm.auto import tqdm
 import csv
 from tfkit.utility.eval_metric import EvalMetric
 from tfkit.utility.model_loader import load_trained_model, load_predict_parameter
@@ -103,8 +103,8 @@ def main(arg=None):
             writer = csv.writer(f)
             records = eval_metric.get_record(eval_arg.get('metric'))
             writer.writerow(['input', 'predicted', 'targets'])
-            for i, p, t in zip(records['input'], records['predicted'], records['targets']):
-                writer.writerow([i, p, tok.tok_sep(model.tokenizer).join([onet for onet in t if len(onet) > 0])])
+            for i, p, t in zip(records['ori_input'], records['ori_predicted'], records['ori_target']):
+                writer.writerow([i, p, t])
         print("write result at:", outfile_name)
 
         with open(outfile_name + "_each_data_score.csv", "w", encoding='utf8') as edsf:
