@@ -9,7 +9,6 @@ dir_path = os.path.dirname(os.path.realpath(__file__))
 sys.path.append(os.path.abspath(os.path.join(dir_path, os.pardir)))
 
 from torch.nn.functional import softmax
-from nlp2 import *
 from tfkit.utility.loss import *
 from tfkit.model.tag.dataloader import get_feature_from_data
 
@@ -32,7 +31,7 @@ class Model(nn.Module):
         self.pretrained = self.pretrained.to(self.device)
         self.loss_fct = self.loss_fct.to(self.device)
 
-    def forward(self, batch_data, eval=False, separator=" "):
+    def forward(self, batch_data, eval=False, separator=" ", **args):
         inputs = batch_data["input"]
         masks = batch_data["mask"]
 
@@ -52,22 +51,16 @@ class Model(nn.Module):
 
             ilogit = softmax(reshaped_logits[0], dim=1)
             result_labels = ilogit.data.tolist()
-            result_items = []
-
-            for pos_logit_prob in result_labels:
-                max_index = pos_logit_prob.index(max(pos_logit_prob))
-                result_items.append(
-                    [self.labels[max_index], dict(zip(self.labels, pos_logit_prob))])
-
-            mapping = batch_data['mapping'][0]
+            token_word_mapping = batch_data['token_word_mapping'][0]
             start, end = batch_data['pos'][0]
-            for map_token in mapping:
-                char, pos = map_token['char'], map_token['pos']
-                if pos > end + 1:  # start from 1
+            for pos, logit_prob in enumerate(result_labels[1:]):  # skip cls and sep
+                if pos >= len(token_word_mapping):
                     break
-                result_dict['label_map'].append({char: result_items[pos - start][0]})
-                result_dict['label_prob_all'].append({char: result_items[pos - start][1]})
-
+                word = token_word_mapping[start + pos]['word']
+                max_index = logit_prob.index(max(logit_prob))
+                result_dict['label_map'].append({word: self.labels[max_index]})
+                result_dict['label_prob_all'].append({word: dict(zip(self.labels, logit_prob))})
+            result_dict['token_word_mapping'] = token_word_mapping[start:end]
             outputs = result_dict
         else:
             targets = batch_data["target"]
@@ -78,7 +71,7 @@ class Model(nn.Module):
         return outputs
 
     def predict(self, input='', neg="O", task=None, handle_exceed='slide',
-                merge_strategy=['minentropy', 'maxprob', 'maxcount'], minlen=1, start_contain="B_", end_contain="I_"):
+                merge_strategy=['minentropy', 'maxprob', 'maxcount'], minlen=1, start_contain="B", end_contain="I"):
         handle_exceed = handle_exceed[0] if isinstance(handle_exceed, list) else handle_exceed
         merge_strategy = merge_strategy[0] if isinstance(merge_strategy, list) else merge_strategy
         self.eval()
@@ -88,21 +81,21 @@ class Model(nn.Module):
             predicted_pos_prob = defaultdict(lambda: defaultdict(list))
             for feature_dict in get_feature_from_data(tokenizer=self.tokenizer, labels=self.labels, input=input.strip(),
                                                       maxlen=self.maxlen, handle_exceed=handle_exceed):
-                start, end = feature_dict['pos']
                 for k, v in feature_dict.items():
                     feature_dict[k] = [v]
                 result = self.forward(feature_dict, eval=True)
-                pos_to_char = feature_dict['mapping'][0]
-                for ind, mapping in enumerate(result['label_prob_all']):
-                    for map_pos, map_dict in mapping.items():
-                        max_label = max(map_dict, key=map_dict.get)
-                        max_prob = map_dict[max_label]
-                        max_entropy = Categorical(probs=torch.tensor(list(map_dict.values()))).entropy().data.tolist()
-                        predicted_pos_prob[str(ind + start)]['char'] = pos_to_char[ind]['char']
-                        predicted_pos_prob[str(ind + start)]['labels'].append(max_label)
-                        predicted_pos_prob[str(ind + start)]['prob'].append(max_prob)
-                        predicted_pos_prob[str(ind + start)]['entropy'].append(max_entropy)
+                for token_pred, token_map in zip(result['label_prob_all'], result['token_word_mapping']):
+                    token_prob = list(token_pred.values())[0]
+                    max_label = max(token_prob, key=token_prob.get)
+                    max_prob = token_prob[max_label]
+                    max_entropy = Categorical(probs=torch.tensor(list(token_prob.values()))).entropy().data.tolist()
+                    predicted_pos_prob[token_map['pos']]['char'] = token_map['word']
+                    predicted_pos_prob[token_map['pos']]['labels'].append(max_label)
+                    predicted_pos_prob[token_map['pos']]['prob'].append(max_prob)
+                    predicted_pos_prob[token_map['pos']]['entropy'].append(max_entropy)
+
                 ret_detail.append(result)
+
             ret_result = []
             for key, value in predicted_pos_prob.items():
                 if merge_strategy == 'maxcount':
@@ -117,15 +110,22 @@ class Model(nn.Module):
 
             output = []
             target_str = ["", ""]
+            after_start = False
             for mapping in ret_result:
                 for k, y in mapping.items():
-                    if (y is not neg and len(target_str[0]) > 0) or start_contain in y:
+                    if start_contain in y:
+                        after_start = True
+                        if len(target_str[0]) > 0:
+                            if len(target_str[0]) > minlen:
+                                output.append(target_str)
+                            target_str = ["", ""]
+                        target_str[0] += k
+                        target_str[1] = y
+                    elif y is not neg and after_start:
                         target_str[0] += k
                         target_str[1] = y
                     else:
-                        if len(target_str[0]) > minlen and target_str not in output:
-                            output.append(target_str)
-                        target_str = ["", ""]
+                        after_start = False
             if len(target_str[0]) > minlen and target_str not in output:
                 output.append(target_str)
             output = [[ner, tag.replace(start_contain, "").replace(end_contain, "")] for ner, tag in output]
