@@ -4,8 +4,8 @@ import sys
 import nlp2
 import tfkit
 import torch
-from tqdm import tqdm
-from transformers import BertTokenizer, AutoTokenizer, AutoModel, AutoConfig
+from tqdm.auto import tqdm
+from transformers import BertTokenizer, AutoTokenizer, AutoModel, get_linear_schedule_with_warmup
 from torch.utils import data
 from itertools import zip_longest
 import os
@@ -41,7 +41,8 @@ def parse_train_args(args):
     parser.add_argument("--seed", type=int, default=609, help="random seed, default 609")
     parser.add_argument("--worker", type=int, default=8, help="number of worker on pre-processing, default 8")
     parser.add_argument("--grad_accum", type=int, default=1, help="gradient accumulation, default 1")
-    parser.add_argument('--tensorboard', dest='tensorboard', action='store_true', help='Turn on tensorboard graphing')
+    parser.add_argument('--tensorboard', action='store_true', help='Turn on tensorboard graphing')
+    parser.add_argument('--wandb', action='store_true', help='Turn on wandb with project name')
     parser.add_argument("--resume", help='resume training')
     parser.add_argument("--cache", action='store_true', help='cache training data')
     parser.add_argument("--panel", action='store_true', help="enable panel to input argument")
@@ -53,19 +54,16 @@ def parse_train_args(args):
     return input_arg, model_arg
 
 
-def optimizer(model, lr):
-    return torch.optim.AdamW(model.parameters(), lr=lr)
+def optimizer(model, lr, total_step):
+    optim = torch.optim.AdamW(model.parameters(), lr=lr)
+    scheduler = get_linear_schedule_with_warmup(optim, num_warmup_steps=int(total_step * 0.05),
+                                                num_training_steps=total_step)
+    return [optim, scheduler]
 
 
 def model_train(models_list, train_dataset, models_tag, input_arg, epoch, logger):
-    optims = []
+    optims_schs = []
     models = []
-    for i, m in enumerate(models_list):
-        model = torch.nn.DataParallel(m)
-        model.train()
-        models.append(model)
-        optims.append(optimizer(m, input_arg.get('lr')[i] if i < len(input_arg.get('lr')) else input_arg.get('lr')[0]))
-
     total_iter = 0
     t_loss = 0
 
@@ -73,9 +71,20 @@ def model_train(models_list, train_dataset, models_tag, input_arg, epoch, logger
     total_iter_length = len(iters[0])
     end = False
     pbar = tqdm(total=total_iter_length)
+
+    for i, m in enumerate(models_list):
+        model = torch.nn.DataParallel(m)
+        model.train()
+        models.append(model)
+        optims_schs.append(
+            optimizer(m, input_arg.get('lr')[i] if i < len(input_arg.get('lr')) else input_arg.get('lr')[0],
+                      total_iter_length))
+
     while not end:
-        for (model, optim, mtag, batch) in zip(models, optims, models_tag, iters):
+        for (model, optim_sch, mtag, batch) in zip(models, optims_schs, models_tag, iters):
             train_batch = next(batch, None)
+            optim = optim_sch[0]
+            scheduler = optim_sch[1]
             if train_batch is not None:
                 loss = model(train_batch)
                 loss = loss / input_arg.get('grad_accum')
@@ -83,8 +92,9 @@ def model_train(models_list, train_dataset, models_tag, input_arg, epoch, logger
                 if (total_iter + 1) % input_arg.get('grad_accum') == 0:
                     optim.step()
                     model.zero_grad()
-                t_loss += loss.mean().item()
-                logger.write_metric("loss/step", loss.mean().item(), epoch)
+                    scheduler.step()
+                t_loss += loss.mean().detach()
+                logger.write_metric("loss/step", loss.mean().detach(), epoch)
                 if total_iter % 100 == 0 and total_iter != 0:  # monitoring
                     logger.write_log(
                         f"epoch: {epoch}, tag: {mtag}, model: {model.module.__class__.__name__}, step: {total_iter}, loss: {t_loss / total_iter if total_iter > 0 else 0}, total:{total_iter_length}")
@@ -115,7 +125,7 @@ def model_eval(models, test_dataset, fname, input_arg, epoch, logger):
                 if test_batch is not None:
                     loss = model(test_batch)
                     loss = loss / input_arg.get('grad_accum')
-                    t_loss += loss.mean().item()
+                    t_loss += loss.mean().detach()
                     t_length += 1
                     pbar.update(1)
                 else:
@@ -184,7 +194,8 @@ def save_model(models, input_arg, models_tag, epoch, fname, logger):
 def main(arg=None):
     input_arg, model_arg = parse_train_args(sys.argv[1:]) if arg is None else parse_train_args(arg)
     nlp2.get_dir_with_notexist_create(input_arg.get('savedir'))
-    logger = Logger(savedir=input_arg.get('savedir'), tensorboard=input_arg.get('tensorboard'))
+    logger = Logger(savedir=input_arg.get('savedir'), tensorboard=input_arg.get('tensorboard', False),
+                    wandb=input_arg.get('wandb', False))
     device = 'cuda' if torch.cuda.is_available() else 'cpu'
     nlp2.set_seed(input_arg.get('seed'))
 
@@ -228,7 +239,7 @@ def main(arg=None):
         ds.increase_with_sampling(train_ds_maxlen)
     for ds in test_dataset:
         ds.increase_with_sampling(test_ds_maxlen)
-
+    logger.write_config(input_arg)
     logger.write_log("TRAIN PARAMETER")
     logger.write_log("=======================")
     [logger.write_log(str(key) + " : " + str(value)) for key, value in input_arg.items()]
@@ -237,10 +248,12 @@ def main(arg=None):
     train_dataset = [data.DataLoader(dataset=ds,
                                      batch_size=input_arg.get('batch'),
                                      shuffle=True,
+                                     pin_memory=True,
                                      num_workers=input_arg.get('worker')) for ds in train_dataset]
     test_dataset = [data.DataLoader(dataset=ds,
                                     batch_size=input_arg.get('batch'),
-                                    shuffle=True,
+                                    shuffle=False,
+                                    pin_memory=True,
                                     num_workers=input_arg.get('worker')) for ds in test_dataset]
 
     # loading model back
